package com.ruoyi.file.service.impl;

import com.ruoyi.file.domain.FileUpdateInfo;
import com.ruoyi.file.mapper.FileUpdateInfoMapper;
import com.ruoyi.file.service.FileUpdateInfoService;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Service
public class FileUpdateInfoServiceImpl implements FileUpdateInfoService {

    @Autowired
    private FileUpdateInfoMapper fileUpdateInfoMapper;

    @Value("${file.remote-url}")
    private String remoteUrlBase;

    @Override
    public void addFileToDatabase(FileUpdateInfo fileUpdateInfo) throws Exception {
        String filePath = fileUpdateInfo.getFilePath(); // 使用新的 getFilePath 方法
        File file = new File(filePath);

        if (file.exists()) {
            // 如果文件存在，计算本地文件的 MD5 值
            String fileMd5 = DigestUtils.md5Hex(new FileInputStream(file));
            fileUpdateInfo.setFileMd5(fileMd5);
            fileUpdateInfo.setUpdateStatus("待检查文件版本");
        } else {
            // 如果文件不存在，设置状态为“待更新”
            fileUpdateInfo.setFileMd5(null);
            fileUpdateInfo.setUpdateStatus("待更新");
        }

        fileUpdateInfo.setVersion("v1.0.0"); // 初始化版本号
        fileUpdateInfo.setUpdateTime(new Date());
        fileUpdateInfoMapper.insert(fileUpdateInfo);
    }

    //检查是否需要更新
    @Override
    public boolean checkFileUpdate(Long id) throws Exception {
        FileUpdateInfo fileUpdateInfo = fileUpdateInfoMapper.selectById(id);
        if (fileUpdateInfo == null) {
            throw new FileNotFoundException("文件信息未找到：id=" + id);
        }

        String localFilePath = fileUpdateInfo.getFilePath(); // 使用新的 getFilePath 方法
        File localFile = new File(localFilePath);

        String remoteFileMd5 = getRemoteFileMd5(fileUpdateInfo.getFileName());

        if (!localFile.exists()) {
            // 如果本地文件不存在，设置状态为“待更新”
            fileUpdateInfo.setRemoteMd5(remoteFileMd5);
            fileUpdateInfo.setUpdateStatus("待更新");
            fileUpdateInfoMapper.updateById(fileUpdateInfo);
            return true;
        } else {
            // 如果本地文件存在，比较 MD5 值
            String localFileMd5 = DigestUtils.md5Hex(new FileInputStream(localFile));
            boolean isUpdateRequired = !localFileMd5.equals(remoteFileMd5);
            fileUpdateInfo.setRemoteMd5(remoteFileMd5);
            fileUpdateInfo.setUpdateStatus(isUpdateRequired ? "待更新" : "无需更新");
            fileUpdateInfoMapper.updateById(fileUpdateInfo);
            return isUpdateRequired;
        }
    }

    //下载更新
    @Override
    public void performFileUpdate(Long id) throws Exception {
        FileUpdateInfo fileUpdateInfo = fileUpdateInfoMapper.selectById(id);
        if (fileUpdateInfo == null) {
            throw new FileNotFoundException("文件信息未找到：id=" + id);
        }

        String localFilePath = fileUpdateInfo.getFilePath(); // 使用新的 getFilePath 方法
        String fileName = fileUpdateInfo.getFileName();
        String remoteUrl = remoteUrlBase + fileName;

        try {
            // 获取 JAR 文件所在的目录
            String serviceDir = FileUpdateInfo.getServiceDir();
            String backupDirPath = serviceDir + File.separator + "backup";
            File backupDir = new File(backupDirPath);
            if (!backupDir.exists()) {
                boolean created = backupDir.mkdirs();
                if (!created) {
                    throw new IOException("无法创建备份目录: " + backupDirPath);
                }
            }

            File originalFile = new File(localFilePath);
            if (originalFile.exists()) {
                String timestamp = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
                String backupFilePath = backupDirPath + File.separator + fileName + "." + timestamp;
                Files.move(originalFile.toPath(), Paths.get(backupFilePath), StandardCopyOption.REPLACE_EXISTING);
                System.out.println("旧文件已备份到: " + backupFilePath);
            }

            downloadFileFromRemote(remoteUrl, localFilePath);

            String newFileMd5 = DigestUtils.md5Hex(new FileInputStream(localFilePath));
            fileUpdateInfo.setFileMd5(newFileMd5);
            fileUpdateInfo.setUpdateTime(new Date());
            fileUpdateInfo.setUpdateStatus("已更新");

            // 递增版本号
            String currentVersion = fileUpdateInfo.getVersion();
            String[] versionParts = currentVersion.substring(1).split("\\."); // 去掉 'v' 并拆分版本号
            int major = Integer.parseInt(versionParts[0]);
            int minor = Integer.parseInt(versionParts[1]);
            int patch = Integer.parseInt(versionParts[2]) + 1; // 小版本号加 1
            fileUpdateInfo.setVersion("v" + major + "." + minor + "." + patch);

            fileUpdateInfoMapper.updateById(fileUpdateInfo);
        } catch (FileNotFoundException e) {
            throw new Exception("文件未找到: " + localFilePath + "，原因: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new Exception("文件写入失败: " + localFilePath + "，原因: " + e.getMessage(), e);
        } catch (Exception e) {
            throw new Exception("文件更新失败: " + localFilePath + "，原因: " + e.getMessage(), e);
        }
    }

    private void downloadFileFromRemote(String remoteUrl, String localFilePath) throws Exception {
        URL url = new URL(remoteUrl);
        Path path = Paths.get(localFilePath);
        try {
            Files.copy(url.openStream(), path, StandardCopyOption.REPLACE_EXISTING);
            System.out.println("文件下载成功: " + localFilePath);
        } catch (IOException e) {
            System.err.println("文件下载失败: " + localFilePath + "，原因: " + e.getMessage());
            throw new Exception("文件下载失败: " + localFilePath, e);
        }
    }

    private String getRemoteFileMd5(String fileName) throws Exception {
        String remoteUrl = remoteUrlBase + fileName;
        System.out.println("远程文件 URL: " + remoteUrl);
        URL url = new URL(remoteUrl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestProperty("Cache-Control", "no-cache");
        try (InputStream in = connection.getInputStream()) {
            return DigestUtils.md5Hex(in);
        }
    }

    @Override
    public List<FileUpdateInfo> getAllFiles() {
        return fileUpdateInfoMapper.selectList(null);
    }

    @Override
    public List<FileUpdateInfo> getFilesByFileNameContaining(String fileName) {
        return fileUpdateInfoMapper.selectList(
                new com.baomidou.mybatisplus.core.conditions.query.QueryWrapper<FileUpdateInfo>()
                        .like("file_name", fileName)
        );
    }

    @Override
    public FileUpdateInfo getFileById(Long id) {
        return fileUpdateInfoMapper.selectById(id);
    }

    @Override
    public void deleteFileById(Long id) {
        fileUpdateInfoMapper.deleteById(id);
    }
}